<?php

class Core_Model_CameraSet_CameraSetType1SectionAccuracies extends Talking_Db_Table {

    protected $_name = 'camera_set_type1_section_accuracy';
    protected $_primary = 'camera_set_type1_section_accuracy_id';
    protected $_rowClass = 'Core_Model_CameraSet_CameraSetType1SectionAccuracy';

    /*
     * This function should be called by poller once an hour (see Core_Model_AccuracyPoller).
     * Returns an array of newly calculated count errors or null if nothing was calculated.
     * Limitations: As it is now designed it does not calculate for previous dates if those are missing.
     *  - Possibly add a separate check and function to calculate accuracies on older data?
     */
    public function calculateAccuracies($force_recalc = false)
    {
        $sections = $this->sectionsToCalculate($force_recalc);
        if (!empty($sections)) {
            $accuracy_rows = $this->calculateAccuracyForSections($sections);
        }
        if (isset($accuracy_rows)) {
            return $accuracy_rows;
        } else {
            return null;
        }
    }

    public function getLatestCountErrorBySection($section_id) {
        $select = $this->select();
        $select->where('camera_set_type1_section_id = ?', $section_id)->order(array("camera_set_type1_section_accuracy_id DESC"))->limit(1);
        return $this->fetchAll($select)->current();
    }

    public function getCountErrorBySectionAndDate($section_id, $date)
    {
        $select = $this->select();
        $select->where('camera_set_type1_section_id = ?', $section_id)->where('date = ?', $date);
        return $this->fetchAll($select)->current();
    }

    public function calculateAccuracyForSections($sections_with_date)
    {
        $count_errors = array();
        foreach ($sections_with_date as $section) {
            $first_date = $section["first_date"];
            $last_date = $section["last_date"];
            $calc_date = $section["calc_date"];
            $prev_error = $section["previous_count_error_id"];
            $count_data = array();
            $reversed_camera_ids = array();
            $sectionCameras = $section["section"]->getSectionCameras();
            $countDataTable = new Core_Model_CountData();

            foreach ($sectionCameras as $sectionCamera) {
                $camera = $sectionCamera->getCamera();
                if ($camera->application_id == 1) {
                    if ($sectionCamera->reverse_count) {
                        $reversed_camera_ids[] = $sectionCamera->camera_id;
                    }
                    $countDataTable->addCamera($camera);
                }
            }
            $countDataTable->initDatabaseSelect(true);
            $count_data = $countDataTable->getData("ts1y", $first_date, $last_date);
            $count_error_val = $this->calculateCountError($count_data, $reversed_camera_ids);
            $count_errors[] = array(
                "section_id" => $section["section"]->camera_set_type1_section_id,
                "camera_set_id" => $section["section"]->camera_set_id,
                "count_error" => $count_error_val,
                "previous_count_error_id" => $prev_error
                );
        }
        $accuracy_rows = array();
        foreach ($count_errors as $count_error) {
            $accuracyData = array(
                "camera_set_type1_section_id" => $count_error["section_id"],
                "count_error" => $count_error["count_error"],
                "date" => $calc_date
                );
            $prev_error_id = $count_error["previous_count_error_id"];
            if ($prev_error_id) {
                //forced rewrite
                $select = $this->select();
                $accuracyRow = $this->fetchAll(
                    $select->where('camera_set_type1_section_accuracy_id = ?', $prev_error_id)
                )->current();
                $accuracyRow->count_error = $count_error["count_error"];
            } else {
                $accuracyRow = $this->createRow($accuracyData);
            }
            $accuracyRow->save();
            $accuracy_rows[] = $accuracyRow;

        }
        if (!empty($accuracy_rows)) {
            return $accuracy_rows;
        } else {
            r_log("No new accuracies were calculated.");
            return null;
        }
    }

    public function calculateCountError($countData, $reversed_camera_ids)
    {
        $sumIn = 0;
        $sumOut = 0;
        foreach ($countData as $data) {
            foreach ($data as $key => $value) {
                if ($key != "cnttime") {
                    $key_array = explode("_", $key, 2);
                    $type = $key_array[1];
                    $camera_id = intval(substr($key_array[0], 1));
                    $reversed = in_array($camera_id, $reversed_camera_ids);
                    if ($type == "in") {
                        if (!$reversed) {
                            $sumIn+= $value;
                        } else {
                            $sumOut+= $value;
                        }
                    } elseif ($type == "out") {
                        if (!$reversed) {
                            $sumOut+= $value;
                        } else {
                            $sumIn+= $value;
                        }
                    }
                }
            };
        }
        if ( $sumIn < 1 ) {
            $count_error = 0;
        } else {
            $count_error = abs(1-($sumIn/(($sumIn+$sumOut)/2)));
            $count_error = round($count_error*100, 2); //Convert to percent and round to two digits.
        }
        return $count_error;
    }

    protected function sectionsToCalculate($force_recalc = false)
    {
        $storesDb = new Core_Model_CameraSet_CameraSetType1s;
        $stores = $storesDb->fetchAll();
        $return_sections = array();
        foreach ($stores as $store) {
            $store_sections = $store->getSections();
            r_log($store->camera_set_id, "= store: ");
            foreach ($store_sections as $section) {
                $sectionCameras = $section->getSectionCameras();
                if ($section->name != "") {
                    $section_name = $section->name;
                } else {
                    $section_name= "(Stores section)";
                }
                r_log($section_name, "== section: ");
                $lowest_timestamp = 0;
                /*
                 * Get the "lowest" available timestamp for all cameras within section.
                 * This timestmap must be past the calculation time for calculation to happen.
                 */
                foreach ($sectionCameras as $sectionCamera) {
                    $camera = $sectionCamera->getCamera();
                    if ($camera->application_id == 1 && $camera->last_date !== NULL) {
                        $camera_last_timestamp = strtotime($camera->last_date);
                        if (!$lowest_timestamp) {
                            $lowest_timestamp = $camera_last_timestamp;
                        } elseif ($camera_last_timestamp < $lowest_timestamp) {
                            $lowest_timestamp = $camera_last_timestamp;
                        }
                    }
                }
                if ($lowest_timestamp) {
                    $calc_hour = $store->getDayEndHour(); //run calculation on midnight + store offset (can be negative) (local time).
                    $last_date = new Zend_Date($lowest_timestamp);
                    $last_date->set($calc_hour, Zend_Date::HOUR)
                        ->set(00, Zend_Date::MINUTE)
                        ->set(00, Zend_Date::SECOND);
                    $first_date = clone $last_date;
                    // CountData->getData has inclusive limits.
                    $last_date->sub('15', Zend_Date::MINUTE);
                    // Make sure we are at the previous day.
                    $first_date->sub($calc_hour + 2, Zend_Date::HOUR);

                    $first_date->set($calc_hour, Zend_Date::HOUR)
                        ->set(00, Zend_Date::MINUTE)
                        ->set(00, Zend_Date::SECOND);
                    //r_log(date("Y-m-d H:i:s", $first_date->getTimestamp()));
                    //r_log(date("Y-m-d H:i:s", $last_date->getTimestamp()));
                    /*
                     * If calculation is done after midnight the calculation is valid for the previous date, ie date of firstDate
                     * If the calculation is done before midnight the calculation is valid for the current date, ie date of last_date
                     */
                    if ($calc_hour<12) {
                        $calc_date = date("Y-m-d", $first_date->getTimestamp());
                    } else {
                        $calc_date = date("Y-m-d", $last_date->getTimestamp());
                    }
                    $previous_count_error = $this->getCountErrorBySectionAndDate(
                        $section->camera_set_type1_section_id,
                        $calc_date
                    );
                    $previous_count_error_id = null;
                    if ($previous_count_error && $force_recalc == false) {
                        r_log("Count Error has already been calculated for this section and date - skipping.");
                        continue;
                    } elseif ($previous_count_error && $force_recalc == true) {
                        r_log("Count Error has already been calculated for this section and date but force is true
                            - Previous count error will be overwritten.");
                        $previous_count_error_id = $previous_count_error->camera_set_type1_section_accuracy_id;
                    }
                    $cameras_is_past_hour = intval(date("H", $lowest_timestamp));

                    r_log($calc_hour, "when to calc (local hour): ");
                    r_log($cameras_is_past_hour, "cameras is past (local hour): ");

                    if ($calc_hour <= $cameras_is_past_hour) {
                        r_log($section_name, "Will calculate accuracy for: ");
                        $return_sections[] = array(
                            "section" => $section,
                            "first_date" => $first_date,
                            "last_date" => $last_date,
                            "calc_date" => $calc_date,
                            "previous_count_error_id" => $previous_count_error_id
                            );
                    } else {
                        r_log($section_name, "Camera data not past calculate hour - skipping. ");
                    }
                } else {
                    r_log("Could not find any camera timestamp - No People Counters in section? - skipping.");
                }
            }
        }
        return $return_sections;
    }
}
