<?php

class Core_Model_Folder extends Zend_Db_Table_Row_Abstract
{
    /*
     * Returns the parent group. NULL if root group.
     */
    public function getParent() {
        return  $this->findParentRow('Core_Model_Folders');
    }

    /*
     * Returns a RowSet containing child groups.
     */
    public function getChildren() {
        $table = new Core_Model_Folders();

        return $this->findDependentRowset($table, NULL, $table->select()->order('name'));
    }

    /*
     * Returns the parent company.
     */
    public function getCompany() {
        return $this->findParentRow('Core_Model_Companies');
    }


    /*
     * Sync folder
     */
    public function sync() {
        $this->getUsers();
    }

    /*
     * Helper function. Uses the synced table to return users.
     */ 
    protected function _getUsersSync()
    {
        $groupsUsersSyncDb = new Core_Model_FoldersUsersSync();

        $viewer_rule = $groupsUsersSyncDb->select()->where('permission = "view"');
        $creator_rule = $groupsUsersSyncDb->select()->where('permission = "create"');

        $viewers = array();
        $creators = array();

        foreach($this->findManyToManyRowset('Core_Model_Users',
                                             'Core_Model_FoldersUsersSync',
                                             NULL, NULL, $viewer_rule) as $v)
            $viewers[] = $v;
        
        foreach($this->findManyToManyRowset('Core_Model_Users',
                                                 'Core_Model_FoldersUsersSync',
                                                 NULL, NULL, $creator_rule) as $c)
            $creators[] = $c;

        return array(
            'viewers' => $viewers,
            'creators' => $creators
            );
    }

    /*
     * Synchronize group member database.
     */
    public function _sync($data)
    {
        $groupsUsers = $this->findDependentRowset('Core_Model_FoldersUsersSync');

        foreach($groupsUsers as $gu)
            $gu->delete();

        $groupsUsersDb = new Core_Model_FoldersUsersSync();

        foreach($data['viewers'] as $viewer) {
            $row = $groupsUsersDb->createRow(array(
                                               'group_id' => $this->group_id,
                                               'user_id' => $viewer->user_id,
                                               'permission' => 'view'));

            $row->save();
        }

        foreach($data['creators'] as $creator) {
            $row = $groupsUsersDb->createRow(array(
                                               'group_id' => $this->group_id,
                                               'user_id' => $creator->user_id,
                                               'permission' => 'create'));

            $row->save();
        }

        $this->in_sync = TRUE;
        $this->save();
    }

    /*
     * Returns all users in a group, seperated into viewers and creators.
     */
    public function getUsers($rec = true) {
        $this->refresh();

        if($this->in_sync && $rec) {
            return $this->_getUsersSync();
        }

        $groupsUsersDb = new Core_Model_FoldersUsers();
        
        $group = $this;

        $viewers = array();
        $creators = array();
        while(!is_null($group)) {
            $viewer_rule = $groupsUsersDb->select()->where('permission = "view"');
            $creator_rule = $groupsUsersDb->select()->where('permission = "create"');
            
            foreach($group->findManyToManyRowset('Core_Model_Users',
                                                 'Core_Model_FoldersUsers',
                                                 NULL, NULL, $viewer_rule) as $v)
                $viewers[] = $v;
            
            foreach($group->findManyToManyRowset('Core_Model_Users',
                                                 'Core_Model_FoldersUsers',
                                                 NULL, NULL, $creator_rule) as $c)
                $creators[] = $c;
            
            $group = $group->getParent();

            if($rec == false)
                break;
        }
        
        $creators = array_unique($creators);
        $viewers = array_unique($viewers);
        $viewers = array_diff($viewers, $creators);

        $ret = array(
            'viewers' => $viewers,
            'creators' => $creators
            );

        if($rec)
            $this->_sync($ret);

        return $ret;
    }

    /*
     * Get number of users in group.
     */
    public function getNumberOfUsers() {
        $users = $this->getUsers();

        return count($users['viewers']) + count($users['creators']);
    }

    /*
     * Set in_sync flag to FALSE for this group as well as all subnodes rooted
     * at this group.
     */
    protected function _setOutOfSync() {
        $this->refresh();
        if(!$this->in_sync)
            return;

        $db = new Core_Model_Folders();

        $subGroups = array($this);

        $children = $this->getChildren();

        while(count($children) > 0) {
            $newChildren = array();
            foreach($children as $c) {
                $subGroups[] = $db->find($c->group_id)->current();
                
                foreach($c->getChildren() as $nc)
                    $newChildren[] = $nc;
            }

            $children = $newChildren;
        }

        foreach($subGroups as $sg) {
            $sg->in_sync = FALSE;
            $sg->save();
        }
    }

    /*
     * Add a user to a group. Can be used to change permission if user is already in group.
     */
    public function add($user, $perm = 'view') {
        if(!in_array($perm, array('view', 'create')))
            throw new WebReport_Exception("A user must be added with either 'create' or 'view' permissions.");
        
        $groupsUsersDb = new Core_Model_FoldersUsers();

        $groupUser = $groupsUsersDb->find($this->group_id, $user->user_id)->current();

        if(!$groupUser) {
            $groupUser = $groupsUsersDb->createRow(array(
                                                       'group_id' => $this->group_id,
                                                       'user_id' => $user->user_id,
                                                       'permission' => $perm));
        }

        $groupUser->permission = $perm;

        $groupUser->save();
        $this->_setOutOfSync();
    }

    /*
     * Remove user from a group.
     */
    public function remove($user) {
        $groupsUsersDb = new Core_Model_FoldersUsers();

        $groupUser = $groupsUsersDb->find($this->group_id, $user->user_id)->current();

        if($groupUser)
            $groupUser->delete();

        $this->_setOutOfSync();
        $this->save();
    }

    /*
     * Returns TRUE if $user can edit contents of $this.
     */
    public function canEdit($user) {
        $this->refresh();

        if($user->multi_company)
            return TRUE;

        if($user->administrator && $user->company_id == $this->company_id)
            return TRUE;

        if(!$this->in_sync) {
            $members = $this->getUsers();

            if(in_array($user, $members['creators']))
                return TRUE;

            return FALSE;
        }

        $groupsUsersDb = new Core_Model_FoldersUsersSync();

        $select = $groupsUsersDb->select()
            ->where('user_id = ?', $user->user_id)
            ->where('group_id = ?', $this->group_id)
            ->where('permission = ?', 'create');

        $row = $groupsUsersDb->fetchRow($select);

        if($row)
            return TRUE;

        return FALSE;
    }

        
    /*
     * Returns TRUE if $user can view contents of $this.
     */
    public function canView($user) {
        $this->refresh();

        if($user->multi_company)
            return TRUE;

        if($user->administrator && $user->company_id == $this->company_id)
            return TRUE;

        if(!$this->in_sync) {
            $members = $this->getUsers();

            if(in_array($user, $members['creators']) ||
               in_array($user, $members['viewers']))
                return TRUE;

            return FALSE;
        }

        $groupsUsersDb = new Core_Model_FoldersUsersSync();

        $select = $groupsUsersDb->select()
            ->where('user_id = ?', $user->user_id)
            ->where('group_id = ?', $this->group_id);

        $row = $groupsUsersDb->fetchRow($select);

        if($row)
            return TRUE;

        return FALSE;
    }

    /*
     * Returns all Documents in the group.
     */
    public function getDocuments($type = NULL) {
        if(is_null($type))
            return $this->findDependentRowset('Core_Model_Documents');

        $documentsDb = new Core_Model_Documents();
        $rule = $documentsDb->select()
            ->where('type = ?', $type);

        return $this->findDependentRowset('Core_Model_Documents', NULL, $rule);
    }

    public function translatables(){
        _T("ALL_COMPANIES");
        _T("BACK_TO_ALL_COMPANIES");
    }

    /*
     * Returns the full path to the folder.
     */
    public function getPath()
    {
        $current = $this;
        $path = '/' . $current->name;

        while(!is_null($current->getParent())) {
            $current = $current->getParent();
            $path = '/' . $current->name . $path;
        }

        return $path;
    }
}
