<?php

class Core_Model_Folders extends Talking_Db_Table
{
    protected $_name = 'group';
    protected $_primary = 'group_id';
    protected $_rowClass = 'Core_Model_Folder';

    protected $_dependentTables = array('Core_Model_Documents', 'Core_Model_Folders', 'Core_Model_Companies', 'Core_Model_FoldersUsers', 'Core_Model_FoldersUsersSync');

    protected $_referenceMap    = array(
        'Company' => array(
            'columns'           => array('company_id'),
            'refTableClass'     => 'Core_Model_Companies',
            'refColumns'        => array('company_id')
        ),
        'Parent' => array(
            'columns'           => array('parent_id'),
            'refTableClass'     => 'Core_Model_Folders',
            'refColumns'        => array('group_id')
        )
    );

    public function fetchAllByCompanyId($company_id)
    {
        $select = $this->select();
        $select->where('company_id = ?', $company_id);


        return $this->fetchAll($select);
    }

    public function fetchFolderWithCameraGroup($camera_group_id)
    {
        $select = $this->select();
        $select->where('camera_group_id = ?', $camera_group_id);
        return $this->fetchAll($select);
    }

    /*
     * Fetch all groups that are viewable by a user.
     */
    public function getViewableGroups($user)
    {
        if($user->multi_company)
            return $this->fetchAll();

        if($user->administrator)
            return $this->fetchAllByCompanyId($user->company_id);

        $select = $this->select()
            ->where('in_sync = ?', FALSE);

        $groups = $this->fetchAll($select);
        foreach($groups as $group)
            $group->sync();

        $viewable = array();
        $db = $this->getAdapter();

        $select = $db->select()
            ->from(array('g' => $this->info(Zend_Db_Table::NAME), array('g.group_id')))
            ->where('g.in_sync = ?', TRUE)
            ->join(array('sync' => 'group_user_sync'),
                'sync.group_id = g.group_id')
                ->where('sync.user_id = ?', $user->user_id);

        $groups = $db->fetchAll($select);

        foreach($groups as $group) 
            $viewable[] = $this->find($group->group_id)->current();



        return $viewable;
    }

    /*
     * Fetch all groups that are editable by a user.
     */
    public function getEditableGroups($user, $company_id = false)
    {
        if($user->multi_company)
            return $this->fetchAll();

        if($user->administrator)
            return $this->fetchAllByCompanyId($user->company_id);

        $select = $this->select()
            ->where('in_sync = ?', FALSE);

        $groups = $this->fetchAll($select);
        foreach($groups as $group) 
            $group->sync();

        $editable = array();

        $db = $this->getAdapter();

        $select = $this->select();
        $select->setIntegrityCheck(false);

        $select
            ->from(array('g' => $this->info(Zend_Db_Table::NAME), array('g.group_id')))
            ->where('g.in_sync = ?', TRUE)
            ->join(array('sync' => 'group_user_sync'),
                'sync.group_id = g.group_id')
                ->where('sync.user_id = ?', $user->user_id)
                ->where('sync.permission = ?', 'create');

        if($company_id)
            $select->where('g.company_id = ?', $company_id);

        $groups = $db->fetchAll($select);

        foreach($groups as $group) 
            $editable[] = $this->find($group->group_id)->current();

        return $editable;
    }

    /*
     * Return all groups where company is active.
     */
    public function fetchAllActive($company_id = null)
    {
        $select = $this->select();
        $select->setIntegrityCheck(false);

        $select->from(array('g' => 'group'))
            ->join(array('c' => 'company'), 
            'c.company_id = g.company_id')
            ->where('c.active = 1')
            ->order('UPPER(g.name)');
        if($company_id)
            $select->where('g.company_id = ?', $company_id);

        return $this->fetchAll($select);   
    }

    /*
     * Fetch all groups for a user that should be shown at the root.
     */
    public function getRootGroups($user, $company_id = null) 
    {
        $groups = $this->fetchAllActive($company_id);

        $viewable = array();
        foreach($groups as $group) {
            if($group->canView($user)) {
                $parent = $group->getParent();

                if(is_null($parent) || !$parent->canView($user))
                    $viewable[] = $group;
            }
        }

        return $viewable;

    }

    /*
     * Search folders.
     */
    public function searchFolders($string, $limit=10, $user=NULL)
    {
        if(is_null($user))
            $user = Zend_Auth::getInstance()->getIdentity();

        $likes = array();
        foreach(explode(' ', $string) as $word) {
            $word = trim($word);

            if(strlen($word))
                $likes[] = "(UPPER(g.name) LIKE UPPER('%$word%'))";
        }
        $like = implode(' AND ', $likes);

        $select = $this->select()
            ->from(array('g' => $this->_name), array('*'))
            ->where($like)
            ->limit(10);

        if(!$user->multi_company) {
            if($user->administrator) {
                $select->where('company_id = ?', $user->company_id);
            } else {
                $select->
                    join(array('s' => 'group_user_sync'), 's.group_id = g.group_id', array())->
                    where('user_id = ?', $user->user_id);
            }
        } else {
            $select->join(array('c' => 'company'), 'c.company_id = g.company_id', array());
            $select->where("c.active = 1");
        }


        $res = array();

        foreach($this->fetchAll($select) as $item) {
            $ar = array(
                'type' => 'folder',
                'group_id' => $item->group_id,
                'name' => $item->name
            );

            $res[] = (object)$ar;
        }

        return $res;
    }
}
