<?php

class Core_Model_Protocol_JSON extends Core_Model_Protocol_Abstract implements Core_Model_Protocol_ProtocolInterface
{
    public function init()
    {
        parent::init();
        $this->setId(1);
    }

    private static function massageData($data)
    {
        #Translate some data attributes to things datamanager understands.
        $massage_attributes = array(
            "app_identifier" => "product",
            "folder_identifier" => "username",
            "folder_password" => "password",
            "reporter_name" => "name",
            "device_mac" => "camera_serial",
            "device_regcode" => "camera_regcode",
        );
        foreach ($massage_attributes as $key => $value) {
            if (isset($data[$key])) {
                $data[$value] = $data[$key];
                unset($data[$key]);
            }
        }
        return $data;
    }

    public static function getLastDataDate($data)
    {
        $data = self::massageData($data);
        $camerasDb = Core_Model_Cameras::getInstance();
        $camera = $camerasDb->fetchCamera($data);
        if (isset($camera)) {
            $camera->updateLocalTimes(null, null); //Update last contact
            return $camera->data_last_utc_timestamp;
        }
        return null;
    }

    public static function addData($data, $ignore_mutex=false)
    {
        # Mutex handling must be done here already so that not multiple cameras can be created on high load.
        if (!$ignore_mutex) {
            $mutex = new Core_Model_Mutex();
            $mutex->init(Core_Model_Mutex::MUTEX_MODIFY_CAMERA_TABLES);
        }
        if (!$ignore_mutex && $mutex->acquire() == false) {
                throw new WebReport_Exception("Unable to require semaphore in JSON::addData.");
        }
        $data = self::massageData($data);
        $objects = self::prepareDataAndFetchObjects($data);
        $app = $objects['app'];
        if ($app instanceof Core_Model_Application_EventDataAbstract) {
            $return_val = Core_Model_EventData::addData($objects, new Core_Model_Protocol_JSON);
        } else {
            $return_val = Core_Model_CountData::addDataForDecodedJSON($objects, new Core_Model_Protocol_JSON);
        }
        if (!$ignore_mutex && $mutex->release()==false) {
            throw new WebReport_Exception("Unable to release semaphore in JSON::addData.");
        }
        return $return_val;
    }

    public static function prepareDataAndFetchObjects($data, $app = null, $group = null)
    {
        if (!isset($data)) {
            r_log("no data received");
            throw new WebReport_Exception("Can't add data. No data received");
        }
        if ($data["protocol"] != 1) {
            throw new WebReport_Exception("Can't add data. Unrecognized protocol version.");
        }
        //To allow app to be sent in from import so it does not have to be recreated each time.
        if (!isset($app)) {
            $appClass = "Core_Model_Application_" . $data["product"];
            $app = new $appClass;
        }
        if (!isset($data["camera_name"]) && isset($data["name"])) {
            $data["camera_name"] = $data["name"];
        }
        $camerasDb = Core_Model_Cameras::getInstance();
        $obj = array(
            "name"      => $data['camera_name'],
            "serial"    => $data['camera_serial'],
            "time"      => intval($data['utc_time_end'] + $data['utc_offset']),
            "utc_offset"=> $data['utc_offset'],
            "regcode"   => $data['camera_regcode'],
            "application_id" => $app->getId(),
            "types"     => $data['data_struct']
        );
        if (isset($data['user'])) {
            $obj['username'] = $data['user'];
        } elseif (isset($data['username'])) {
            $obj['username'] = $data['username'];
        }
        if (isset($data['pass'])) {
            $obj['password'] = $data['pass'];
        } elseif (isset($data['password'])) {
            $obj['password'] = $data['password'];
        }
        if ($app instanceof Core_Model_Application_AsEventDataInterface) {
            $app = $app->asEventDataApplication();
            $eventdata = true;
        } else {
            $eventdata = false;
        }
        $camera = $camerasDb->fetchOrCreateCamera($obj, $group);
        $ret['camera'] = $camera;
        $ret['app'] = $app;
        $ret['modified_data'] = $app->letAppModifyData($data);
        if (!empty($data['data'])) { //Only do this id we actually have new data.
            //$ret['data_struct'] = self::perhapsModifyDataStruct($data);
            if ($eventdata) {
                $eventDb = $app->getEventDataTable();
                //To not append on rewrite of data we must delete any present data
                $app->deleteTimespanForCamera($camera, $data['utc_time_start'], $data['utc_time_end']);
            }
        }
        if (isset($eventDb)) {
            $ret['eventDb'] = $eventDb;
        }
        return $ret;
    }

    public static function createLocalCntTimeFromTimestampAndOffset($timestamp, $utc_offset)
    {
        $datetime = self::safelyCreateDateTime($timestamp + $utc_offset);
        return $datetime->format("Y-m-d H:i:s");
    }

    public static function updateTimestamps($camera, $utc_offset, $first_timestamp, $last_timestamp)
    {
        if (isset($last_timestamp)) {
            $camera->updateDataLastUTCTimestamp($last_timestamp);
            $to = self::safelyCreateDateTime($last_timestamp + $utc_offset);
            $camera->updateLocalTimes(null, $to->format('Y-m-d H:i:s'));
        }
        if (isset($first_timestamp)) {
            $from =  self::safelyCreateDateTime($first_timestamp + $utc_offset);
            $camera->updateLocalTimes($from->format('Y-m-d H:i:s'), null);
        }
    }

    public static function safelyCreateDateTime($timestamp)
    {
        $date = DateTime::createFromFormat('U.u', $timestamp);
        if ($date == false) {
            $date = DateTime::createFromFormat('U', $timestamp);
        }
        return $date;
    }

    public static function updateOpeningHours($data, $camera)
    {
        if ($data['utc_time_start']==0) {
            //Special case for Demographics, first time camera reports.
            $from = self::safelyCreateDateTime(time() + $data['utc_offset']);
        } else {
            $from = self::safelyCreateDateTime($data['utc_time_start'] + $data['utc_offset']);
        }
        $to = self::safelyCreateDateTime($data['utc_time_end'] + $data['utc_offset']);
        $interval = DateInterval::createFromDateString('1 day');
        $to->add($interval); #DatePeriod does not include incomplete days, so by adding one day we will also get current day in DatePeriod.
        $period = new DatePeriod($from, $interval, $to);
        $openingHoursDb = Core_Model_OpeningHours::getInstance();
        foreach ($period as $day) {
            $dataDate = $day->format('Y-m-d');
            $weekday_digit = $day->format('w') - 1; #-1 as php Date always refers to Sunday as day 0.
            if ($weekday_digit == -1) {
                $weekday_digit = 6; # Sunday
            }
            $openingHours = $openingHoursDb->fetchAllOpeningHoursForDate($camera->camera_id, $dataDate);
            $no_of_current_rows = sizeof($openingHours);
            $day_enabled = $data["day_enabled"][$weekday_digit];
            if ($day_enabled) {
                $current_day_openings = $data["day_open"][$weekday_digit];
                $current_day_closings = $data["day_close"][$weekday_digit];
                if (sizeof($current_day_openings) != sizeof($current_day_closings)) {
                    r_log("Bad data in schedule. Not equal amounts of opening and closings.");
                    throw new WebReport_Exception("Bad data in schedule. Not equal amount of openings and closings.");
                }
                if (is_string($current_day_openings)) {
                    #To make it iterable.
                    $tmp = array();
                    $tmp[] = $current_day_openings;
                    $current_day_openings = $tmp;
                    $tmp = array();
                    $tmp[] = $current_day_closings;
                    $current_day_closings = $tmp;
                } elseif (!is_array($current_day_openings)) {
                    r_log("Bad data in schedule. Day schedule in bad format.");
                    throw new WebReport_Exception("Bad data in schedule. Day schedule in bad format.");
                }
                $no_of_new_rows = sizeof($current_day_openings);
                for ($i=0; $i<$no_of_new_rows; $i++) {
                    $open = $current_day_openings[$i];
                    $close = $current_day_closings[$i];
                    if ($i<$no_of_current_rows) {
                        $openingHours[$i]->opens = $open;
                        $openingHours[$i]->closes = $close;
                        $openingHours[$i]->save();
                    } else {
                        $openingHour = $openingHoursDb->createRow(
                            array(
                                'camera_id' => $camera->camera_id,
                                'date' => $dataDate,
                                'conn_protocol' => 5, //5 here means it's directly from camera.
                                'opens' => $open,
                                'closes' => $close)
                        );
                        $openingHour->save();
                    }
                }
                for ($j=$no_of_current_rows-1; $j>=$no_of_new_rows; $j--) {
                    $openingHours[$j]->delete();
                }
            } elseif (isset($openingHours)) {
                #Schedule has been disabled for this day.
                for($k=0; $k<$no_of_current_rows; $k++) {
                    $openingHours[$k]->delete();
                }
            }
        }

    }
}
