<?php

abstract class AmCharts_Abstract {

    protected $_settings;
    protected $_data;

    protected $_chartType;

    protected $_view;

    protected $_labels;

    /**
     * Path to AmCharts swf files
     *
     * @var string
     */
    protected static $_swfFilesPath = null;

    /**
     * Path to SWFObject javascript file
     *
     * @var string
     */
    protected static $_swfObjectScriptPath = null;


    public function __construct()
    {
        $className = get_class($this);
        $classNameArr = explode("_", $className);
        $chartType = strtolower($classNameArr[1]);

        $this->_setChartType($chartType);
        
        $settingsClassName = "AmCharts_Settings_" . ucfirst($chartType);

        $this->_setSettingsClass($settingsClassName);

        $dataClassName = "AmCharts_Data_" . ucfirst($chartType);
        
        $this->_setDataClass($dataClassName);

        $this->_labels = new AmCharts_Labels();

        $this->_data->appendLabels($this->_labels);

    }

    /**
     * Setter for chart type, for instance line etc.
     *
     * @param string $chartType
     */
    protected function _setChartType($chartType)
    {
        $this->_chartType = $chartType;
    }

    /**
     * Setter for settings class.
     *
     * @param string|AmCharts_Settings_Abstract $settingsClass Either string with name of settings class or settings.
     */
    protected function _setSettingsClass($settingsClass)
    {
        if (is_string($settingsClass)) {

            try {
                @Zend_Loader::loadClass($settingsClass);
                $this->_settings = new $settingsClass();
            } catch (Zend_Exception $e) {
                throw new AmCharts_Exception("Chart settings don't exist.");
            }
        } else if ($settingsClass instanceof AmCharts_Settings_Abstract) {
            $this->setSettings(new $settingsClass());
        } else {
            throw new AmCharts_Exception("Not valid settings class");
        }
    }

    /**
     * Setter for settings class
     *
     * @param AmCharts_Settings_Abstract $settings
     */
    public function setSettings(AmCharts_Settings_Abstract $settings)
    {
        $this->_settings = $settings;
    }

    /**
     * Getter for settings class
     *
     * @return AmCharts_Settings_Abstract
     */
    public function getSettings()
    {
        return $this->_settings;
    }

    /**
     * Setter for data class.
     *
     * @param string|AmCharts_Settings_Abstract $dataClass Either string with name of data class or settings.
     */
    protected function _setDataClass($dataClass)
    {
        
        if (is_string($dataClass)) {
            try {
                @Zend_Loader::loadClass($dataClass);
                $this->setData(new $dataClass());
            } catch (Zend_Exception $e) {
                throw new AmCharts_Exception("Chart data don't exist.");
            }
        } else if ($dataClass instanceof AmCharts_Data_Abstract) {
            $this->setData(new $dataClass());
        } else {
            throw new AmCharts_Exception("Not valid data class");
        }
    }

    /**
     * Setter for data class
     *
     * @param AmCharts_Data_Abstract $data
     */
    public function setData(AmCharts_Data_Abstract $data)
    {
        $this->_data = $data;
    }

    /**
     * Getter for data class
     *
     * @return AmCharts_Data_Abstract
     */
    public function getData()
    {
        return $this->_data;
    }


    /**
     * Set view object
     *
     * @param  Zend_View_Interface $view
     * @return Zend_Form
     */
    public function setView(Zend_View_Interface $view = null)
    {
        $this->_view = $view;
        return $this;
    }

    /**
     * Retrieve view object
     *
     * If none registered, attempts to pull from ViewRenderer.
     *
     * @return Zend_View_Interface|null
     */
    public function getView()
    {
        if (null === $this->_view) {
            require_once 'Zend/Controller/Action/HelperBroker.php';
            $viewRenderer = Zend_Controller_Action_HelperBroker::getStaticHelper('viewRenderer');
            $this->setView($viewRenderer->view);
        }

        return $this->_view;
    }

    /**
     * Function that get called before rendering for last minute modifications.
     */
    protected function _preRender()
    {}

    /**
     *
     * @param <type> $id
     * @param <type> $width
     * @param <type> $height
     * @param <type> $bgColor
     * @return bool True if data exists, else false
     */
    public function render($id, $width = "100%", $height = "100%", $bgColor = "#FFFFFF")
    {
        
        if (!self::$_swfFilesPath)
            throw new AmCharts_Exception(".swf files path set not set.");

        if (!self::$_swfObjectScriptPath)
            throw new AmCharts_Exception("SWFObject file path not set");

        $this->_preRender();

        $view = $this->getView();

        $view->headScript()->appendFile(self::$_swfObjectScriptPath);



        $view->headScript()->captureStart(); ?>
jQuery(document).ready(function() {
    var so = new SWFObject("<?= self::$_swfFilesPath; ?>/am<?= $this->_chartType; ?>.swf", "amline", "<?= $width; ?>", "<?= $height; ?>", "8", "<?= $bgColor; ?>");
    so.addVariable("path", "<?= self::$_swfFilesPath; ?>/");
    so.addVariable("chart_settings", "<?= $this->_settings; ?>");
    so.addVariable("chart_data", "<?= str_replace("\"", "'", $this->_data); ?>");
    so.addParam("wmode", "opaque");
    so.write("<?= $id; ?>");
});

<?php
        $view->headScript()->captureEnd();


        return $this->dataExists();

    }

    /**
     *
     * @return bool True if data exist, else false
     */
    public function dataExists()
    {
        if(is_null($this->_data->getXLabels())) return False;
        return ((count($this->_data->getXLabels()->getTags())) > 0);
    }


    public function addLabel($text, $x = 0, $y = 0, $params = array())
    {
        $this->_labels->addLabel($text, $x, $y, $params);
    }

    public function getLabels()
    {
        return $this->_labels;
    }

    /**
     * Set path to SWF Object javascript file.
     *
     * @param string $path Set path to SWF Object javascript file.
     */
    public static function setSwfObjectScriptPath($path)
    {
        $path = Talking_Filesystem_File::getAbsolutePath($path, true);
        self::$_swfObjectScriptPath = $path;
    }

    /**
     * Set path to AmCharts swf files.
     *
     * @param string $path Path to AmCharts swf files.
     */
    public static function setSwfFilesPath($path)
    {
        $path = Talking_Filesystem_File::getAbsolutePath($path, true);
        self::$_swfFilesPath = $path;
    }

    public function renderData(){
        return str_replace("\"", "'", $this->_data);
    }


}
