<?php

set_include_path(get_include_path() . PATH_SEPARATOR . dirname(__file__) . '/phpseclib');

include('Crypt/RSA.php');

class Settings {
    private static $a = array();

    public static function load() {
        $settings = json_decode(file_get_contents(dirname(__file__) . '/settings.json'), TRUE);

        foreach(array_keys($settings) as $key)
            self::$a[$key] = $settings[$key];
    }

    public static function get($key) {
        return self::$a[$key];
    }
}

Settings::load();

/*
 * Generate RSA key pair.
 */
function gen_keys() {
    $rsa = new Crypt_RSA();

    $keys = $rsa->createKey(Settings::get('RSA_NUMBITS'));

    return array(
        'public' => $keys['publickey'],
        'private' => $keys['privatekey']
    );
}

/*
 * Return a JSON representation in canonical form.
 */
function canonical($ob)
{
    if(is_float($ob))
        throw new InvalidArgumentException('Floating point numbers not allowed due to rounding errors.');
    else if(is_string($ob))
        return sprintf('"%s"', $ob);
    else if(is_int($ob))
        return (string) $ob;
    else if(is_array($ob)) {
        if($ob == array_values($ob))
            return '[' . implode(',', array_map("canonical", $ob)) . ']';
        else
            $keys = array_keys($ob);
        sort($keys);

        $l = array();
        foreach($keys as $key) {
            if(!is_string($key))
                throw new InvalidArgumentException('Only string keys allowed.');
            $l[] = sprintf('"%s":%s', $key, canonical($ob[$key]));
        }

        return '{' . implode(',', $l) . '}';
    } else
        throw new InvalidArgumentException('Only string, integers, lists and dictionaries allowed.');
}

/*
 * An RSA-signed message.
 */
class Message
{
    public $data;
    public $sig;

    function __construct($data, $sig = NULL) {
        $this->data = $data;
        $this->sig = $sig;

        try {
            canonical($data);
        } catch(InvalidArgumentException $e) {
            throw new InvalidArgumentException('Data must be serializable.');
        }
    }

    /*
     * The string that is signed.
     */
    public function sigString() {
        return utf8_encode(canonical($this->data));
    }

    /*
     * Verify message using a public key. Returns a boolean.
     */
    public function verify($pubkey) {
        if(is_null($this->sig))
            throw new BadMethodCallException('Cannot verify unsigned data.');

        if(!is_string($pubkey))
            throw new InvalidArgumentException('Not a valid RSA key.');

        $rsa = new Crypt_RSA();
        $rsa->loadKey($pubkey);
        $rsa->setSignatureMode(CRYPT_RSA_SIGNATURE_PKCS1);

        if($rsa->verify($this->sigString(), base64_decode($this->sig)))
            return TRUE;

        return FALSE;
    }

    /*
     * Sign a message using a private key.
     */
    public function sign($privkey) {
        if(!is_string($privkey))
            throw new InvalidArgumentException('Not a valid RSA key.');

        $rsa = new Crypt_RSA();
        $rsa->loadKey($privkey);
        $rsa->setSignatureMode(CRYPT_RSA_SIGNATURE_PKCS1); 

        $this->sig = base64_encode($rsa->sign($this->sigString()));
    }

    /*
     * Dump message as a JSON string.
     */
    public function saveJson() {
        // NOTE: PHP 5.3 doesn't support pretty print.
        return json_encode($this);
    }

    /*
     * Load a signed message from a JSON string.
     */
    public static function loadJson($ob) {
        $d = json_decode($ob, TRUE);

        return new Message($d['data'], $d['sig']);
    }
}

?>
