<?php
class Talking_Pdf_Page extends Zend_Pdf_Page
{

    /**
     * Align text at left of provided coordinates
     * 
     * @var string
     */
    const TEXT_ALIGN_LEFT = 'left';
    
    /**
     * Align text at right of provided coordinates
     * 
     * @var string
     */
    const TEXT_ALIGN_RIGHT = 'right';
    
    /**
     * Center-text horizontally within provided coordinates
     * 
     * @var string
     */
    const TEXT_ALIGN_CENTER = 'center';
	
	
    /**
     * Extension of basic draw-text function to allow it to vertically center text
     *
     * @param string $text
     * @param int $x1
     * @param int $y1
     * @param int $x2
     * @param int $position
     * @param string $encoding
     * @return Zend_Pdf_Page
     */
    public function drawAlignedText($text, $x1, $y1, $x2 = null, $position = self::TEXT_ALIGN_LEFT, $encoding = null){
        if( $encoding == null ) $encoding = Talking_Pdf::$encoding;

        $bottom = $y1; // could do the same for vertical-centering
        switch ($position) {
        case self::TEXT_ALIGN_LEFT :
                $left = $x1;
                break;
            case self::TEXT_ALIGN_RIGHT :
            	if (null === $x2) {
                    throw new Exception ( "Cannot right-align text horizontally, x2 is not provided" );
                }
                $textWidth = $this->getTextWidth ( $text );
                $left = $x2 - $textWidth;
                break;
            case self::TEXT_ALIGN_CENTER :
                if (null === $x2) {
                    throw new Exception ( "Cannot center text horizontally, x2 is not provided" );
                }
                $textWidth = $this->getTextWidth ( $text );
                $left = $x1 + $textWidth / 2;

                break;
            default :
                throw new Exception ( "Invalid position value \"$position\"" );
        }

        // display multi-line text
        $this->drawText ( $text, $left, $y1, $encoding );
    }
    
    /**
     * Draw text inside a box using word wrap
     * 
     * @param string $text
     * @param int $x1
     * @param int $y1
     * @param int $x2
     * @param int $position
     * @param float $lineHeight
     * @param string $encoding
     * 
     * @return integer bottomPosition
     */
    public function drawTextBox($text, $x1, $y1, $x2 = null, $position = self::TEXT_ALIGN_LEFT, $lineHeight = 1.3, $encoding = null){
        if( $encoding == null )  {
        		$encoding = Talking_Pdf::$encoding;
        	}
        		

        $lines = explode(PHP_EOL, $text);
        
        $bottom = $y1;
        $lineHeight = $this->getFontSize() * $lineHeight;
        foreach( $lines as $line ){
            preg_match_all('/([^\s]*\s*)/i', $line, $matches);
            
            $words = $matches[1];
            
            $lineText = '';
            $lineWidth = 0;
            $prevLine = '';
            foreach( $words as $word ){
                $lineText .= $word;
                $lineWidth = $this->getTextWidth($lineText);
                $url_regex = "#((http|https|ftp)://(\S*?\.\S*?))(\s|\;|\)|\]|\[|\{|\}|,|\"|'|:|\<|$|\.\s)#ie";

                if(preg_match($url_regex, $word)) {
                    // $word is URL create link

                    $target = Zend_Pdf_Action_URI::create($word);
                    $linkWidth = $this->getTextWidth($word);

                    if($lineWidth >= $x2-$x1) {
                      $annotation = Zend_Pdf_Annotation_Link::create(
                        $x1,
                        $bottom,
                        $x1 + $linkWidth,
                        $bottom - $lineHeight,
                        $target
                      );
                    } 
                    else {
                        $annotation = Zend_Pdf_Annotation_Link::create(
                          $x1 + $lineWidth - $linkWidth, 
                          $bottom + $lineHeight,
                          $x1 + $lineWidth, 
                          $bottom,
                          $target);
                    }

                    $this->attachAnnotation($annotation);
                }

                if( $lineWidth >= $x2-$x1 ){
                    $this->drawAlignedText($prevLine, $x1, $bottom, $x2, $position, $encoding );
                    $bottom -= $lineHeight;
                    $lineText = $word;
                    $line = '';
                    $lineWidth = $this->getTextWidth($word);
                }
                $prevLine = $lineText;
            }

            $this->drawAlignedText( $lineText, $x1, $bottom, $x2, $position, $encoding );
            $bottom -= $lineHeight;
        }
        
        return $bottom;
    }
    
    
    /**
     * Return length of generated string in points
     *
     * @param string                     $text
     * @param Zend_Pdf_Resource_Font|Zend_Pdf_Page     $font
     * @param int                         $fontSize
     * @return double
     */
	     
	function getTextWidth($string, $font = null, $fontSize = null)
	{
	        if( !isset($encoding) || $encoding == null ) $encoding = Talking_Pdf::$encoding;
	        
	        if( !$font ){
	            $font = $this->getFont();
	            $fontSize = $this->getFontSize();
	        } elseif( $font instanceof Zend_Pdf_Resource_Font ){
	            if( $fontSize === null ) throw new Talking_Pdf_Exception('The fontsize is unknown');
	        }
	        
	        if( !$font instanceof Zend_Pdf_Resource_Font ){
	            throw new Exception('Invalid resource passed');
	        }	
	
	     $drawingString = iconv('UTF-8', 'UTF-16BE//IGNORE', $string);
	     $characters = array();
	     for ($i = 0; $i < strlen($drawingString); $i++) {
	         $characters[] = (ord($drawingString[$i++]) << 8 ) | ord($drawingString[$i]);
	     }
	     $glyphs = $font->glyphNumbersForCharacters($characters);
	     $widths = $font->widthsForGlyphs($glyphs);
	     $stringWidth = (array_sum($widths) / $font->getUnitsPerEm()) * $fontSize;
	     return $stringWidth;
	 }

}
