<?php

class WebReport_FileFormats_Cnt {


	/* options */
	protected $_verbose = false;
	protected $_logger;
	protected $_onlyValidateHeader = false;

	/* file specific */
	protected $_filename;
	protected $_version;
	protected $_blocks = array();

	protected $_valid = true;


	public function __construct($filename, $options = array()) {

        if (!is_array($options)) {
            if ($options instanceof Zend_Config) {
                $options = $options->toArray();
            } else {
                throw new Talking_Exception('Options must be in an array or a Zend_Config object');
            }
        }


		if (!file_exists($filename)) {
			throw new WebReport_Exception("File $filename not exists");
		}


		if (!is_readable($filename)) {
			throw new WebReport_Exception("File $filename not readable");
		}

		$this->_filename = $filename;

        if (array_key_exists(WebReport_FileFormats::VERBOSE, $options)) {
            $this->_verbose = (bool) $options[WebReport_FileFormats::VERBOSE];
        }

		if (array_key_exists(WebReport_FileFormats::ONLY_VALIDATE, $options)) {
			$this->_onlyValidateHeader = $options[WebReport_FileFormats::ONLY_VALIDATE];
		}

        if (array_key_exists(WebReport_FileFormats::LOGGER, $options)) {
			if (!($options[WebReport_FileFormats::LOGGER] instanceof Zend_Log)) {
				throw new WebReport_Exception("Not valid logger");
			}
			if ($this->_verbose) {
				$this->_logger = $options[WebReport_FileFormats::LOGGER];
			} else {
				$logger = new Zend_Log();
				$logger->addWriter(new Zend_Log_Writer_Null());
				$this->_logger = $logger;
			}
        } else {

			$logger = new Zend_Log();
			if ($this->_verbose) {
				$logger->addWriter(new Zend_Log_Writer_Stream('php://stdout'));
			} else {
				$logger->addWriter(new Zend_Log_Writer_Null());
			}
			$this->_logger = $logger;
		}

		if (!$this->_onlyValidateHeader) {
			try {
				$this->_parse();
			} catch (WebReport_FileFormats_Exception_InvalidFile $e) {
				$this->_valid = false;
			}
		}

	}


	protected function _parse() {
		$logger = $this->_logger;

		$handle = fopen($this->_filename, "rb");
		$data = fread($handle, 6);

		/* read header */
		if ($data != 'CogCNT') {
			throw new WebReport_FileFormats_Exception_InvalidFile("Invalid file, header: $data");
		}

		$firstRead = true;

		$currentBlock = null;
		$currentBlockCntSet = null;

		$i = 0;

		while (!feof($handle)) {
			$i++;
			// read eventual code
			$data = fread($handle, 2);
			if (!feof($handle)) {
				$code = current(unpack("s",$data));
				if ($this->_verbose and $code < 0) $logger->notice("Code: " . $code);
				if ($code == -2 || $firstRead) { // new block
					if ($firstRead) {
						fseek($handle, -2, SEEK_CUR);
					}
					$firstRead = false;

					$block = new WebReport_FileFormats_Cnt_Block();

					$data = fread($handle, 1);
					$version = current(unpack("C",$data));

					if ($version != 6 && $version != 5 && $version != 4 && $version != 2) {
						throw new WebReport_FileFormats_Exception_InvalidFile("Invalid version : $version");
					}


					$block->setVersion($version);

					$data = fread($handle, 2);
					$nbrOfTypes = current(unpack("v",$data));
					$this->_nbrOfTypes = $nbrOfTypes;

					$data = fread($handle, 2*$nbrOfTypes);
					$types = unpack("v" . $nbrOfTypes,$data);
					$block->setTypes($types);

					$data = fread($handle, 2);
					$nameByteSize = current(unpack("v",$data));

					// read name
					$data = fread($handle, 1*$nameByteSize);
					$nameArr = unpack("C" . $nameByteSize, $data);
					$name = '';
					foreach ($nameArr as $c) {
						$name .= chr($c);
					}

					$block->setName($name);

					$logger->notice("Version: " . $block->getVersion());
					$logger->notice("Number of types: " . $block->getNumberOfTypes());
					$logger->notice("Types: " . implode(", ", $block->getTypes()));
					$logger->notice("Name: " . $block->getName());

					$currentBlock = $block;
					$this->addBlock($block);

				} elseif ($code == -1) { // new cntDataBlock on current block

					$cntSet = new WebReport_FileFormats_Cnt_Block_CntSet();
					$currentBlockCntSet = $cntSet;
					$currentBlock->addCntSet($cntSet);

					$data = fread($handle, 4);
					$startTime = current(unpack("V",$data));

					// read delta
					$data = fread($handle, 4);
					$delta = current(unpack("V",$data));

					// read key
					$data = fread($handle, 4);
					$key = current(unpack("V",$data));

					$cntSet->setStartTime($startTime);
					$cntSet->setDelta($delta);
					$cntSet->setKey($key);

					$logger->notice("readnew cntdataset: ");
					$logger->notice("Starttime: " . date('Y-m-d H:i:s', $cntSet->getStartTime()));
					$logger->notice("Delta: " . $cntSet->getDelta() . ' ms = ' . $cntSet->getDelta()/(1000*60) . ' min');
					$logger->notice("Key: " . $cntSet->getKey());
				} else { // new cntDataLine
					fseek($handle, -2, SEEK_CUR);
					$types = $currentBlock->getTypes();
					$typesData = array();
					foreach ($types as $type) {
						// read code
						$data = fread($handle, 2);
						$typesData[$type] = current(unpack("v",$data)) . " ";
					}

					if ($this->_verbose) {
						$logger->notice("[" . implode(", ", array_keys($typesData)) . "]: " . implode(", ", $typesData));
					}
					$currentBlockCntSet->addData($typesData);
				}
			}
		}

		fclose($handle);
    }

    
	public function isValid()
	{

		$handle = fopen($this->_filename, "rb");
		$data = fread($handle, 6);

		$i = 0;
		$version = fread($handle, 1);
		$version = current(unpack("C",$version));
              
		fclose($handle);
                $ret = true;
		if ($data != 'CogCNT') {
			$ret = false;
		}
		if ($version != 6 && $version != 5 && $version != 4 && $version != 2) {
			$ret = false;
		}
                //Zend_Debug::dump(array($data,$version,$ret));
		return $ret;
	}


	public function addBlock(WebReport_FileFormats_Cnt_Block $block)
	{
		$this->_blocks[] = $block;
	}

	public function getBlocks()
	{
		return $this->_blocks;
	}

}
